/*
* Copyright (c) 2011 Massimo Cavalleri <massimo.cavalleri@gmail.com>
* Copyright (c) 2011 Mattia Andena <progmat2000@yahoo.it>
* Copyright (c) 2011 Alessandro Ceseno <ing@alexceseno.it>
*
* This file is part of TTEI.
*
* TTEI is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* TTEI is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with TTEI.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* <p>Title: Filtri antirumore </p>
* <p>Description:Medio, Mediano, Nagano, Gauss
* <p>Copyright: Copyright (c) 2011</p>
* <p>License: GPL </p>
* @author Massimo Cavalleri, Mattia Andena, Alessandro Ceseno
* @version 1.0
*/
import java.util.*;
public class Fr{
	
  private int w,h;
  
  public Fr(int width, int height) {

    this.w = width;
    this.h = height;

  }
  
  public void chiamataFr(int[] in,int[] out, int indf, String mv )
  {
  	//in questo modo in[] e out[] possono essere la stessa matrice!
  	int intmp[]=new int [h*w];
  	for(int i=0;i<h;i++)
      for(int j=0;j<w;j++)
       intmp[i*w+j]=in[i*w+j];
       
  	switch(indf)
  	{
  		
  		case 1:Medio(intmp,out);
               break; 		
   		case 2:Mediano(intmp,out);
  		       break;
  		case 3:Nagao(intmp,out);
  		       break;
  		case 4:Gauss(intmp,out,mv);
  	}
  }
  
   public void Medio(int[] in,int[] out) {
    int A,B,C,D,E,F,G,H;
    for(int i=0;i<h;i++)
      for(int j=0;j<w;j++)
       out[i*w+j]=0;


     for(int i=1;i<h-1;i++)
         for(int j=1;j<w-1;j++)
         {
           A = in[(i-1)*w+(j-1)];
	       B = in[(i-1)*w+j];
	       C = in[(i-1)*w+(j+1)];
	       D = in[i*w+(j-1)];
	       E = in[i*w+(j+1)];
	       F = in[(i+1)*w+(j-1)];
	       G = in[(i+1)*w+j];
	       H = in[(i+1)*w+(j+1)];
	       
           //attua la media degli intorni del punto corrente (incluso lo stesso) e la sovrascrive al punto corrente stesso     
	   out[i*w+j]=(A+B+C+D+E+F+G+H+in[i*w+j])/9; 
         }                                        



   }


   public void Mediano(int[] in, int[] out)
   {

    int Sort[]=new int[9];
      for(int i=0;i<h;i++)
      for(int j=0;j<w;j++)
       {out[i*w+j]=0;
       }//chiude for

     for(int i=1;i<h-1;i++)
         for(int j=1;j<w-1;j++)
         {
               Sort[0]= in[(i-1)*w+(j-1)];
	       Sort[1]= in[(i-1)*w+j];
	       Sort[2]= in[(i-1)*w+(j+1)];
	       Sort[3]= in[i*w+(j-1)];
	       Sort[4]= in[i*w+(j+1)];
	       Sort[5]= in[(i+1)*w+(j-1)];
	       Sort[6]= in[(i+1)*w+j];
	       Sort[7]= in[(i+1)*w+(j+1)];
               Sort[8]= in[i*w+j];
           //Ordina il vettore in ordine crescente 
	   Arrays.sort(Sort);
           
           //assegna al punto corrente del vettore d'uscita il valor medio (situato in posizione centrale del vettore Sort ordinato) 
	   out[i*w+j]=Sort[4];
         }
   }
   public void Nagao(int[] in, int[] out)
   {
     int A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y;
     int Medie[]=new int[9];
     int Vari[]=new int[9];
     
     for(int i=0;i<h;i++)
      for(int j=0;j<w;j++)
       out[i*w+j]=0;
       

     for(int i=2;i<h-2;i++)
         for(int j=2;j<w-2;j++)
         {
           A= in[(i-2)*w+(j-2)];
           B= in[(i-2)*w+(j-1)];
           C= in[(i-2)*w+j];
           D= in[(i-2)*w+(j+1)];
           E= in[(i-2)*w+(j+2)];
           F= in[(i-1)*w+(j-2)];
           G= in[(i-1)*w+(j-1)];
	       H =in[(i-1)*w+j];
	       I= in[(i-1)*w+(j+1)];
	       J= in[(i-1)*w+(j+2)];
           K= in[i*w+(j-2)];
           L= in[i*w+(j-1)];
	       M= in[i*w+j];
           N= in[i*w+(j+1)];
	       O= in[i*w+(j+2)];
           P= in[(i+1)*w+(j-2)];
           Q= in[(i+1)*w+(j-1)];
	       R= in[(i+1)*w+j];
	       S= in[(i+1)*w+(j+1)];
           T= in[(i+1)*w+(j+2)];
           U= in[(i+2)*w+(j-2)];
           V= in[(i+2)*w+(j-1)];
           W= in[(i+2)*w+j];
           X= in[(i+2)*w+(j+1)];
           Y= in[(i+2)*w+(j+2)];
           
           //Determina le medie dei nove intorni considerati
	   Medie[0]=(B+C+D+G+H+I+M)/7;
           Medie[1]=(F+G+K+L+M+P+Q)/7;
           Medie[2]=(M+Q+R+S+V+W+X)/7;
           Medie[3]=(I+J+M+N+O+S+T)/7;
           Medie[4]=(A+B+F+G+H+I+M)/7;
           Medie[5]=(D+E+H+I+J+M+N)/7;
           Medie[6]=(L+M+P+Q+R+U+V)/7;
           Medie[7]=(M+N+R+S+T+X+Y)/7;
           Medie[8]=(G+H+I+L+M+N+Q+R+S)/9;
           
           //Determina le varianze dei nove intorni considerati
	   Vari[0]=(int) (Math.pow((B-Medie[0]),2)+(Math.pow(C-Medie[0],2))+(Math.pow(D-Medie[0],2))+(Math.pow(G-Medie[0],2))+(Math.pow(H-Medie[0],2))+(Math.pow(I-Medie[0],2))+(Math.pow(M-Medie[0],2)))/7;
           Vari[1]=(int) (Math.pow((F-Medie[1]),2)+(Math.pow(G-Medie[1],2))+(Math.pow(K-Medie[1],2))+(Math.pow(L-Medie[1],2))+(Math.pow(M-Medie[1],2))+(Math.pow(P-Medie[1],2))+(Math.pow(Q-Medie[1],2)))/7;
           Vari[2]=(int) (Math.pow((M-Medie[2]),2)+(Math.pow(Q-Medie[2],2))+(Math.pow(R-Medie[2],2))+(Math.pow(S-Medie[2],2))+(Math.pow(V-Medie[2],2))+(Math.pow(W-Medie[2],2))+(Math.pow(X-Medie[2],2)))/7;
           Vari[3]=(int) (Math.pow((I-Medie[3]),2)+(Math.pow(J-Medie[3],2))+(Math.pow(M-Medie[3],2))+(Math.pow(N-Medie[3],2))+(Math.pow(O-Medie[3],2))+(Math.pow(S-Medie[3],2))+(Math.pow(T-Medie[3],2)))/7;
           Vari[4]=(int) (Math.pow((A-Medie[4]),2)+(Math.pow(B-Medie[4],2))+(Math.pow(F-Medie[4],2))+(Math.pow(G-Medie[4],2))+(Math.pow(H-Medie[4],2))+(Math.pow(I-Medie[4],2))+(Math.pow(M-Medie[4],2)))/7;
           Vari[5]=(int) (Math.pow((D-Medie[5]),2)+(Math.pow(E-Medie[5],2))+(Math.pow(H-Medie[5],2))+(Math.pow(I-Medie[5],2))+(Math.pow(J-Medie[5],2))+(Math.pow(M-Medie[5],2))+(Math.pow(N-Medie[5],2)))/7;
           Vari[6]=(int) (Math.pow((L-Medie[6]),2)+(Math.pow(M-Medie[6],2))+(Math.pow(P-Medie[6],2))+(Math.pow(Q-Medie[6],2))+(Math.pow(R-Medie[6],2))+(Math.pow(U-Medie[6],2))+(Math.pow(V-Medie[6],2)))/7;
           Vari[7]=(int) (Math.pow((M-Medie[7]),2)+(Math.pow(N-Medie[7],2))+(Math.pow(R-Medie[7],2))+(Math.pow(S-Medie[7],2))+(Math.pow(T-Medie[7],2))+(Math.pow(X-Medie[7],2))+(Math.pow(Y-Medie[7],2)))/7;
           Vari[8]=(int) (Math.pow((G-Medie[8]),2)+(Math.pow(H-Medie[8],2))+(Math.pow(I-Medie[8],2))+(Math.pow(L-Medie[8],2))+(Math.pow(M-Medie[8],2))+(Math.pow(N-Medie[8],2))+(Math.pow(Q-Medie[8],2))+(Math.pow(R-Medie[8],2))+(Math.pow(S-Medie[8],2)))/9;

           //Controlla quale delle nove varianze si avvicini di pi al punto corrente(M)
	   int ind=0,vicinoM = (Math.abs(Vari[0]-M));
           for(int v=1;v<9;v++)
             if( (Math.abs(Vari[v]-M)) < vicinoM )
               {vicinoM = (Math.abs(Vari[v]-M)); 
                ind=v;
               }
           
           //Assegna al punto corrente della matrice d'uscita il valore della media la cui varianza si avvicina di + al punto corrente della
	   //matrice  
	   out[i*w+j] = Medie[ind]; 

        }

   }
   
   public void Gauss(int[] in, int[] out, String mv)
   {
   	int A,B,C,D,E,F,G,H,i1,i2,i3,i4;
   	
   	if(mv.equals("matrice"))
   	 {
   	  for(int i=1;i<h-1;i++)
       for(int j=1;j<w-1;j++)
       {
	      A = in[(i-1)*w+(j-1)];
	      B = in[(i-1)*w+j];
	      C = in[(i-1)*w+(j+1)];
	      D = in[i*w+(j-1)];
	      E = in[i*w+(j+1)];
	      F = in[(i+1)*w+(j-1)];
	      G = in[(i+1)*w+j];
	      H = in[(i+1)*w+(j+1)];
	      
	      out[i*w+j]= (A+4*B+C+4*D+12*in[i*w+j]+4*E+F+4*G+H)/32;
       }
   	 }
   	else //vettore
   	{
   	  int in2[] = new int[h*w]; 
   	  //orizzontale
   	  for(int i=0;i<h;i++)
       for(int j=2;j<w-2;j++)
       {
       	  i1 = in[i*w+(j-2)];
	      i2 = in[i*w+(j-1)];
	      i3 = in[i*w+(j+1)];
	      i4 = in[i*w+(j+2)];
	   
	      in2[i*w+j]= (4*(i2+i4)+i1+i4+6*in[i*w+j])/16;
       }
      
      
       //verticale
      for(int i=2;i<h-2;i++)
       for(int j=0;j<w;j++)
       {
       	  i1 = in2[(i-2)*w+j];
	      i2 = in2[(i-1)*w+j];
	      i3 = in2[(i+1)*w+j];
	      i4 = in2[(i+2)*w+j];
	      
	      out[i*w+j]= (4*(i2+i4)+i1+i4+6*in2[i*w+j])/16;
       }
   	}
//Normalizzazione   	
   }
   
}
