/*
* Copyright (c) 2011 Massimo Cavalleri <massimo.cavalleri@gmail.com>
* Copyright (c) 2011 Mattia Andena <progmat2000@yahoo.it>
* Copyright (c) 2011 Alessandro Ceseno <ing@alexceseno.it>
* Copyright (c) 2003 Luca Lombardi <luca.lombardi@unipv.it>
* Copyright (c) 2003 Paolo Lombardi <paolo.lombardi@unipv.it>
*
* This file is part of TTEI.
*
* TTEI is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* TTEI is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with TTEI.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * <p>Title: Visualizzatore di immagini </p>
 * <p>Description: Scheletro per il corso di elaborazione di immagini </p>
 * <p>Copyright: Copyright (c) 2003, 2011</p>
 * <p>License: GPL </p>
 * <p>Company: Laboratorio di Visione Artificiale </p>
 * @author Luca Lombardi, Paolo Lombardi, Massimo Cavalleri, Mattia Andena, Alessandro Ceseno
 * @version 1.0
 */

import java.awt.*;
import java.awt.image.*;
import java.applet.Applet;


public class ImageView extends Applet 
{

  static final int DEFAULT_WIDTH = 384;
  static final int DEFAULT_HEIGHT = 288;
  static final int DEFAULT_WIN_SIZE = 15;
  static final int MAX_WIN_SIZE = 15;
  protected Dimension image_size = new Dimension(DEFAULT_WIDTH, DEFAULT_HEIGHT);

  protected Image image;
  protected Image result_image;

  protected int[] inputpx;

  protected Image zoom_window;
  protected int window_size = DEFAULT_WIN_SIZE;

  protected ImageViewCanvas image_canvas;
  protected ImageViewCanvas result_canvas;
  protected ImageCanvas zoom_canvas;
  protected Choice zoom_choice;
  protected Choice zoom_choiceMF;
  protected Choice zoom_choiceFr;
  protected Choice zoom_choiceMV;
  protected Choice zoom_choiceN;
  protected Choice zoom_choiceG;
  protected Choice zoom_choiceF;
  protected Choice zoom_choiceNR;
  protected Choice zoom_choiceOut;
  protected Label sample_info;
  protected Panel north_panel;
  protected Panel south_panel;
  
  //-------------------------------------------------------------------------------------
  
  public void loadImage(String image_file,int w,int h, int inpx[])
	{
	 //*GetImage:*
	 //Returns an Image object that can then be painted on the screen. 
     //The url argument must specify an absolute URL. 
     //The name argument is a specifier that is relative to the url argument.
     //*GetDocumentBase:*
     //Returns an absolute URL naming the directory of the document in which 
     //the applet is embedded. 
     
     Image imageL;
     int[] inputpxL;
     
	 imageL = getImage(getDocumentBase(), image_file);
	 inputpxL = new int[h*w];
	
	 
	 PixelGrabber pg2 = new PixelGrabber(imageL, 0, 0, w, h, inputpxL, 0, w);

     try {
      
        pg2.grabPixels();
    
     }
     catch (InterruptedException e) {
      
        System.err.println("ImageView-loadImage: interrupted while grabbing pixels");

     }
     if ((pg2.status() & ImageObserver.ABORT) != 0) {
      
        System.err.println("ImageView-loadImage: pixel grab aborted or errored");
   
     }
	 
	 this.toGrayOneByte(inputpxL, inpx);	
  
  }

 //-----------------------------------------------------------------------------------------

  private boolean grabImage(int[] data) {
    PixelGrabber pg = new PixelGrabber(image, 0, 0, image_size.width, image_size.height, data, 0, image_size.width);

    try {
      pg.grabPixels();
    }
    catch (InterruptedException e) {
      System.err.println("ImageView: interrupted while grabbing pixels");
      return false;
    }
    if ((pg.status() & ImageObserver.ABORT) != 0) {
      System.err.println("ImageView: pixel grab aborted or errored");
      return false;
    }

    return true;

  }

  private void toGrayOneByte(int[] in, int[] out)
  {
    for(int i=0; i<in.length; i++) {
      int red = (in[i] & 0x00ff0000) >> 16;
      int green = (in[i] & 0x0000ff00) >> 8;
      int blue = (in[i] & 0x000000ff);
      out[i] = ((int) (red * 0.35 + green * 0.45 + blue *0.20));
    }
  }

  private void toGrayThreeBytes(int[] in, int[] out)
  {
    for(int i=0; i<in.length; i++) {
      out[i] = ((in[i] & 255) * 0x00010101) | 0xff000000;
    }
  }

  public void init() {

    // Initiate image loading.

    String image_file = getParameter("IMAGEFILE");
    if (image_file == null) {
      image_file = "pd1.gif";
      System.out.println("ImageView: using default image file");
    }
    image = getImage(getDocumentBase(), image_file);

    // Determine preferred dimensions for image canvas.

    
    String width_string = getParameter("IMAGEWIDTH");
    if (width_string != null) {
      try {
        image_size.width = Integer.parseInt(width_string);
      }
      catch (NumberFormatException e) {
        image_size.width = DEFAULT_WIDTH;
      }
    }
    
    System.out.println(image_size.width);

    String height_string = getParameter("IMAGEHEIGHT");
    if (height_string != null) {
      try {
        image_size.height = Integer.parseInt(height_string);
      }
      catch (NumberFormatException e) {
        image_size.height = DEFAULT_HEIGHT;
      }
    }
    
    System.out.println(image_size.height);
    // Initialise standard components

    
    inputpx = new int[image_size.width * image_size.height];


    // Grab and display default input image

    grabImage(inputpx);

    MemoryImageSource source;
    source = new MemoryImageSource(image_size.width, image_size.height, inputpx, 0, image_size.width);

    result_image = createImage(source);


    // Create user interface components.

    image_canvas = new ImageViewCanvas(image, image_size, window_size, MAX_WIN_SIZE, this);
    sample_info = new Label("[999,999] 111:111:111");
    sample_info.setAlignment(Label.CENTER);
    sample_info.setForeground(Color.red);

    zoom_window = createImage(MAX_WIN_SIZE, MAX_WIN_SIZE);
    Dimension zoom_size = new Dimension(10*MAX_WIN_SIZE, 10*MAX_WIN_SIZE);
    zoom_canvas = new ImageCanvas(image, zoom_size);
    result_canvas = new ImageViewCanvas(result_image, image_size, window_size, MAX_WIN_SIZE, this);

    zoom_choice = new Choice();
    zoom_choice.addItem("Original Image");
    zoom_choice.addItem("Invert");
    zoom_choice.addItem("Sobel");
    zoom_choice.addItem("Prewitt");
    zoom_choice.addItem("Isotropico");
    zoom_choice.addItem("Kirsch");
    zoom_choice.addItem("Roberts");
    zoom_choice.addItem("DoG");
    zoom_choice.select("Original Image");

    zoom_choiceMF = new Choice();
    zoom_choiceMF.addItem("modulo");
    zoom_choiceMF.addItem("fase");
    zoom_choiceMF.select("modulo");
    zoom_choiceFr = new Choice();
    zoom_choiceFr.addItem("Original Image");
    zoom_choiceFr.addItem("Medio");
    zoom_choiceFr.addItem("Mediano");
    zoom_choiceFr.addItem("Nagao");
    zoom_choiceFr.addItem("Gaussiano");
    zoom_choiceFr.select("Original Image");
    
    zoom_choiceMV = new Choice();
    zoom_choiceMV.addItem("matrice");
    zoom_choiceMV.addItem("vettore");
    zoom_choiceMV.select("matrice");
    
    zoom_choiceN = new Choice();
    zoom_choiceN.addItem("1");
    zoom_choiceN.addItem("2");
    zoom_choiceN.addItem("3");
    zoom_choiceN.addItem("4");
    zoom_choiceN.addItem("5");
    zoom_choiceN.select("1");
    zoom_choiceN.setEnabled(false);

    zoom_choiceG = new Choice();
    zoom_choiceG.addItem("Nessuna");
    zoom_choiceG.addItem("Rette");
    zoom_choiceG.addItem("Cerchi Piccoli");
    zoom_choiceG.addItem("Cerchi Medi");
    zoom_choiceG.addItem("Cerchi Grandi");
    zoom_choiceG.addItem("Triangoli");
    zoom_choiceG.addItem("Quadrati Piccoli");
    zoom_choiceG.addItem("Quadrati Medi");
    zoom_choiceG.addItem("Quadrati Grandi");
    zoom_choiceG.addItem("Rettangoli");
    zoom_choiceG.addItem("Esagoni Piccoli");
    zoom_choiceG.addItem("Esagoni Grandi");
    zoom_choiceG.addItem("Ottagoni");
    zoom_choiceG.select("Nessuna");
    
    zoom_choiceNR = new Choice();
    zoom_choiceNR.addItem("30");
    zoom_choiceNR.addItem("25");
    zoom_choiceNR.addItem("20");
    zoom_choiceNR.addItem("15");
    zoom_choiceNR.select("30");
    
    zoom_choiceF = new Choice();
    zoom_choiceF.addItem("Nessuna");
    zoom_choiceF.addItem("pinzatrice (S)");
    zoom_choiceF.addItem("pinzatrice (D)");
    zoom_choiceF.addItem("mouse");
    zoom_choiceF.addItem("gomma");
    zoom_choiceF.addItem("spillatrice"); 
    zoom_choiceF.select("Nessuna");
    
    zoom_choiceOut = new Choice();
    zoom_choiceOut.addItem("immagine di uscita");
    zoom_choiceOut.addItem("spazio dei parametri"); 
    zoom_choiceOut.select("immagine di uscita");
   
   
    north_panel = new Panel();
    north_panel.setLayout(new BorderLayout(10, 10));
    north_panel.add("Center", image_canvas);
    north_panel.add("East", result_canvas);

    south_panel = new Panel();
    south_panel.setLayout(new BorderLayout(30, 10));
    south_panel.add("Center", zoom_canvas);

    Panel info_panel = new Panel();
    Panel info_panel_p=new Panel();
    
    info_panel_p.setLayout(new BorderLayout(5, 5));
    info_panel.setLayout(new GridLayout(4, 4, 10, 10));//riempie per righe
    
    Label fr = new Label("Filtri anti-rumore:", Label.CENTER);
    fr.setForeground(Color.blue);
    Label op = new Label("Operatori:", Label.CENTER);
    op.setForeground(Color.blue);
    Label h = new Label("Hough:", Label.CENTER);
    h.setForeground(Color.blue);
    
    //1 linea
    info_panel.add(fr);
    info_panel.add(op);
    info_panel.add(h);
    info_panel.add(new Label(""));
    
    //2 linea
    info_panel.add(zoom_choiceFr);
    info_panel.add(zoom_choice);
    info_panel.add(zoom_choiceG);
    Panel NRpanel = new Panel();
    NRpanel.setLayout(new GridLayout(1, 2, 10, 10));
    NRpanel.add(new Label("n linee:",Label.CENTER));
    NRpanel.add(zoom_choiceNR);
    info_panel.add(NRpanel);
    
    //3 linea
    info_panel.add(zoom_choiceMV);
    info_panel.add(zoom_choiceMF);
    info_panel.add(zoom_choiceF);
    info_panel.add(new Label(""));
    
    //4 linea
    Panel Npanel = new Panel();
    Npanel.setLayout(new GridLayout(1, 2, 10, 10));
    Npanel.add(new Label("n volte:",Label.CENTER));
    Npanel.add(zoom_choiceN);
    
    info_panel.add(Npanel);
    
    info_panel.add(new Label(""));
    info_panel.add(zoom_choiceOut);
    //end line
    
    info_panel_p.add("North",sample_info);
    info_panel_p.add("South",info_panel);
    
    zoom_choiceMF.setEnabled(false);
    zoom_choiceMV.setEnabled(false);
    zoom_choiceOut.setEnabled(false);
    zoom_choiceNR.setEnabled(false);
    
    south_panel.add("East", info_panel_p);

    setBackground(Color.lightGray);
    setLayout(new FlowLayout(FlowLayout.CENTER, 15, 10));
    add(north_panel);
    add(south_panel);

    validate();
    sample_info.setText("");

  }


  public void destroy() {
    zoom_window.flush();
    image.flush();
  }


  public void newSample(Point p, int value) {
    int r = (value & 0xff0000) >> 16;
    int g = (value & 0x00ff00) >> 8;
    int b = value & 0x0000ff;
    sample_info.setText("[" + p.x + "," + p.y + "]  " + r + ":" + g + ":" + b);
  }


  public void newZoom(int[] data) {
    Image img = createImage(new MemoryImageSource(window_size, window_size, data, 0, MAX_WIN_SIZE));
    zoom_canvas.newImage(img);
  }

  public boolean action(Event event, Object arg) {
  	
   int[] invpx = new int[image_size.width * image_size.height];
   int[] dummypx = new int[image_size.width * image_size.height];
   System.out.println("e' accaduto evento");
   
      if((!(zoom_choiceG.getSelectedItem().equals("Nessuna")))&&(zoom_choiceF.getSelectedItem().equals("Nessuna")))  
       {
       	zoom_choice.select("Original Image");
       	zoom_choiceFr.select("Original Image");
       	zoom_choiceN.select("1");
        zoom_choiceMV.select("matrice");
        zoom_choiceMF.select("modulo");
        zoom_choiceF.select("Nessuna");
        
       	zoom_choiceN.setEnabled(false);
        zoom_choiceMF.setEnabled(false);
        zoom_choiceMV.setEnabled(false);
        zoom_choiceNR.setEnabled(false);
        zoom_choice.setEnabled(false);
        zoom_choiceFr.setEnabled(false);
        zoom_choiceF.setEnabled(false);
        
        
        // passo necessario: converte i pixel da 3 a 1 byte
        this.toGrayOneByte(inputpx, invpx);

        if ( (zoom_choiceG.getSelectedItem()).equals("Rette") )
        {
            System.out.println("retta");
            
           zoom_choiceNR.setEnabled(true);
           zoom_choiceOut.select("immagine di uscita");
           zoom_choiceOut.setEnabled(false);
           
           int nline = Integer.parseInt(zoom_choiceNR.getSelectedItem());
           Retta b=new Retta(image_size.width,image_size.height);
           b.hough(invpx,dummypx,nline);
           
           if(arg.equals("Rette"))
            b.disegnaSp();
        }
        else
        {
        	System.out.println("poligoni");
        	zoom_choiceOut.setEnabled(true);
             
             Poligoni b=new Poligoni(image_size.width,image_size.height);
             b.hough(invpx,dummypx,(zoom_choiceOut.getSelectedItem()),(zoom_choiceG.getSelectedIndex())-2);
        }
        

        // riconverte a 3 byte per la rappresentazione sul canvas
        this.toGrayThreeBytes(dummypx, invpx);

        MemoryImageSource source;
        source = new MemoryImageSource(image_size.width, image_size.height, invpx, 0, image_size.width);

        result_image = createImage(source);
        result_canvas.newImage(result_image);
        
        return true;
      }
      
      if((!(zoom_choiceF.getSelectedItem().equals("Nessuna")))&&(zoom_choiceG.getSelectedItem().equals("Nessuna")))  
       {
       	zoom_choice.select("Original Image");
       	zoom_choiceFr.select("Original Image");
       	zoom_choiceN.select("1");
        zoom_choiceMV.select("matrice");
        zoom_choiceMF.select("modulo");
        zoom_choiceG.select("Nessuna");
        
       	zoom_choiceN.setEnabled(false);
        zoom_choiceMF.setEnabled(false);
        zoom_choiceMV.setEnabled(false);
        zoom_choice.setEnabled(false);
        zoom_choiceFr.setEnabled(false);
        zoom_choiceG.setEnabled(false);
        zoom_choiceOut.setEnabled(true);
        zoom_choiceNR.setEnabled(false);
        
        // passo necessario: converte i pixel da 3 a 1 byte
        this.toGrayOneByte(inputpx, invpx);

        System.out.println("sono nella parte giusta");
        
        Oggetto O = new Oggetto(invpx, image_size.width, image_size.height);
        
        if( (zoom_choiceF.getSelectedItem()).equals("pinzatrice (S)"))
        {
          int Modello[] = new int[74*165];
          loadImage("pinzatriceS.gif",74,165,Modello);
          O.hough(Modello,74,165,dummypx,(zoom_choiceOut.getSelectedItem()));
        }
        
        if( (zoom_choiceF.getSelectedItem()).equals("mouse"))
        {
          int Modello[] = new int[113*105];
          loadImage("mouse.gif",105,113,Modello);
          O.hough(Modello,105,113,dummypx,(zoom_choiceOut.getSelectedItem()));
        } 

        if( (zoom_choiceF.getSelectedItem()).equals("gomma"))
        {
          int Modello[] = new int[85*49];
          loadImage("gomma.gif",85,49,Modello);
          O.hough(Modello,85,49,dummypx,(zoom_choiceOut.getSelectedItem()));	
        }
        
        if( (zoom_choiceF.getSelectedItem()).equals("pinzatrice (D)"))
        {
          int Modello[] = new int[85*173];
          loadImage("pinzatriceD.gif",85,173,Modello);
          O.hough(Modello,85,173,dummypx,(zoom_choiceOut.getSelectedItem()));	
        }
        
        if( (zoom_choiceF.getSelectedItem()).equals("spillatrice"))
        {
          int Modello[] = new int[85*137];
          loadImage("spillatrice.gif",85,137,Modello);
          O.hough(Modello,85,137,dummypx,(zoom_choiceOut.getSelectedItem()));	
        }

        // riconverte a 3 byte per la rappresentazione sul canvas
        this.toGrayThreeBytes(dummypx, invpx);

        MemoryImageSource source;
        source = new MemoryImageSource(image_size.width, image_size.height, invpx, 0, image_size.width);

        result_image = createImage(source);
        result_canvas.newImage(result_image);
        
        return true;
      }
     
       if((zoom_choiceG.getSelectedItem().equals("Nessuna"))&&(zoom_choiceF.getSelectedItem().equals("Nessuna")))
        {   
          zoom_choice.setEnabled(true);
          zoom_choiceFr.setEnabled(true);
          zoom_choiceG.setEnabled(true);
          zoom_choiceF.setEnabled(true);
          zoom_choiceOut.setEnabled(false);
          zoom_choiceNR.setEnabled(false);
        	
      	  //N.B. no return true!!!
        }

  	  if( (zoom_choice.getSelectedItem().equals("Original Image")) && (zoom_choiceFr.getSelectedItem().equals("Original Image")) ) 
       {
        MemoryImageSource source;
        source = new MemoryImageSource(image_size.width, image_size.height, inputpx, 0, image_size.width);
        result_image = createImage(source);
        result_canvas.newImage(result_image);
        
        zoom_choiceN.setEnabled(false);
        zoom_choiceMF.setEnabled(false);
        zoom_choiceMV.setEnabled(false);
        zoom_choiceNR.setEnabled(false);
          
        return true;
       
       }
       
      
  
  	  if((zoom_choiceFr.getSelectedItem() != "Original Image")&&(zoom_choice.getSelectedItem().equals("Original Image")))
	   {
        Fr elaborateR = new Fr(image_size.width, image_size.height);
        
        zoom_choiceN.setEnabled(true);
        
        zoom_choiceMF.setEnabled(false);
        
        if(zoom_choiceFr.getSelectedIndex()==4) 
          zoom_choiceMV.setEnabled(true);
        else
          zoom_choiceMV.setEnabled(false);
          
          // passo necessario: converte i pixel da 3 a 1 byte
          this.toGrayOneByte(inputpx, invpx);  
          
        for(int i=0;i<image_size.height;i++)
          for(int j=0;j<image_size.width;j++)
	     	dummypx[i*image_size.width+j]=invpx[i*image_size.width+j];
	          
            
        if((zoom_choiceMV.getSelectedItem()).equals("matrice"))
	     {
	     	
	     	for(int i=0;i<zoom_choiceN.getSelectedIndex()+1;i++)
	         elaborateR.chiamataFr(dummypx, dummypx, zoom_choiceFr.getSelectedIndex(),"matrice"); 
	     }
	    else
	     {
	     	
	     	for(int i=0;i<zoom_choiceN.getSelectedIndex()+1;i++)
	         elaborateR.chiamataFr(dummypx, dummypx, zoom_choiceFr.getSelectedIndex(),"vettore"); 
	     }
	    
	     // elabora e riconverte a 3 byte per la rappresentazione sul canvas
	     this.toGrayThreeBytes(dummypx, invpx);
	    
	  MemoryImageSource source;
      source = new MemoryImageSource(image_size.width, image_size.height, invpx, 0, image_size.width);
      result_image = createImage(source);
      result_canvas.newImage(result_image);
      
      return true;

	 }
	    
    if((zoom_choice.getSelectedItem() != "Original Image")&&(zoom_choiceFr.getSelectedItem().equals("Original Image")))
    {
        zoom_choiceN.setEnabled(false);
        	
        if ( zoom_choice.getSelectedItem().equals("Invert") || zoom_choice.getSelectedItem().equals("DoG") ) 
          zoom_choiceMF.setEnabled(false);
        else  
           zoom_choiceMF.setEnabled(true); 
      
        Op elaborate = new Op(image_size.width,image_size.height);
        
        int tipo;
	    String mf = new String();
	    tipo = zoom_choice.getSelectedIndex();
	    mf = zoom_choiceMF.getSelectedItem();
	     
	    // passo necessario: converte i pixel da 3 a 1 byte
        this.toGrayOneByte(inputpx, invpx);
	       
	    elaborate.chiamataOp(invpx, dummypx, tipo, mf);
      
      // elabora e riconverte a 3 byte per la rappresentazione sul canvas     
	    this.toGrayThreeBytes(dummypx, invpx);
      
     MemoryImageSource source;
     source = new MemoryImageSource(image_size.width, image_size.height, invpx, 0, image_size.width);
     result_image = createImage(source);
     result_canvas.newImage(result_image);
     
     return true;
     }
     
     
  
 
  if( (zoom_choice.getSelectedItem() != "Original Image") && (zoom_choiceFr.getSelectedItem() != "Original Image") ) 
  {  //tutti e due hanno sel un filtro o operatore => quindi vanno eseguiti concatenati 
  	 
  	 int[] dummypxTmp = new int[image_size.width * image_size.height];
  	 
	   Fr elaborateR = new Fr(image_size.width, image_size.height);
	   
	    zoom_choiceN.setEnabled(true);
        
        if(zoom_choiceFr.getSelectedIndex()==4) 
          zoom_choiceMV.setEnabled(true);
        else
          zoom_choiceMV.setEnabled(false);
          
          // passo necessario: converte i pixel da 3 a 1 byte
          this.toGrayOneByte(inputpx, invpx);  
            
        for(int i=0;i<image_size.height;i++)
          for(int j=0;j<image_size.width;j++)
	     	dummypxTmp[i*image_size.width+j]=invpx[i*image_size.width+j];
	          
            
        if((zoom_choiceMV.getSelectedItem()).equals("matrice"))
	     {
	     	
	     	for(int i=0;i<zoom_choiceN.getSelectedIndex()+1;i++)
	         elaborateR.chiamataFr(dummypxTmp, dummypxTmp, zoom_choiceFr.getSelectedIndex(),"matrice"); 
	     }
	    else
	     {
	     	
	     	for(int i=0;i<zoom_choiceN.getSelectedIndex()+1;i++)
	         elaborateR.chiamataFr(dummypxTmp, dummypxTmp, zoom_choiceFr.getSelectedIndex(),"vettore"); 
	     }
	    
	     // elabora e riconverte a 3 byte per la rappresentazione sul canvas
	     this.toGrayThreeBytes(dummypxTmp, invpx);
	     
	   
      	// passo necessario: converte i pixel da 3 a 1 byte
        this.toGrayOneByte(dummypxTmp, invpx);

        Op elaborate = new Op(image_size.width,image_size.height);
        
        if ( zoom_choice.getSelectedItem().equals("Invert") || zoom_choice.getSelectedItem().equals("DoG") ) 
          zoom_choiceMF.setEnabled(false);
        else  
           zoom_choiceMF.setEnabled(true);
           
	    int tipo;
	    String mf = new String();
	    tipo = zoom_choice.getSelectedIndex();
	    mf = zoom_choiceMF.getSelectedItem();
	       
	    elaborate.chiamataOp(invpx, dummypx, tipo, mf);
	    
        // elabora e riconverte a 3 byte per la rappresentazione sul canvas     
	    this.toGrayThreeBytes(dummypx,invpx);

	  MemoryImageSource source;
      source = new MemoryImageSource(image_size.width, image_size.height, invpx, 0, image_size.width);
      result_image = createImage(source);
      result_canvas.newImage(result_image);
	 	
   return true;
  }
  
   return false;//nessun evento verificato
  
 }
 
 
}




class ImageViewCanvas extends ImageCanvas {


  protected ImageView controller;
  protected Point cursor_pos;
  protected Point window_pos;
  protected int window_size;
  protected int max_size;
  protected int margin;
  protected int[] data;


  public ImageViewCanvas(Image img, Dimension dim, int window_size,
   int max_size, ImageView controller) {
    super(img, dim);
    this.controller = controller;
    cursor_pos = new Point(0, 0);
    window_pos = new Point(0, 0);
    this.window_size = window_size;
    this.max_size = max_size;
    margin = window_size/2;
    data = new int[max_size*max_size];
  }


  public void paint(Graphics g) {
    g.drawImage(image, 0, 0, getBackground(), this);
  }


  public void markPosition() {
    Graphics g = this.getGraphics();
    paint(g);
    g.setColor(Color.red);
    g.drawRect(window_pos.x-margin, window_pos.y-margin, window_size, window_size);
  }


  public boolean sampleWindow(int win_size) {
    int m = win_size/2;
    if (window_pos.x >= m && window_pos.x < size.width - m &&
        window_pos.y >= m && window_pos.y < size.height - m) {
      window_size = win_size;
      margin = m;
      markPosition();
      return sampleWindow();
    }
    else
      return false;
  }

  public boolean sampleWindow() {

    PixelGrabber pg = new PixelGrabber(image, window_pos.x - margin,
      window_pos.y - margin, window_size, window_size, data, 0, max_size);

    try {
      pg.grabPixels();
    }
    catch (InterruptedException e) {
      System.err.println("ImageView: interrupted while grabbing pixels");
      return false;
    }
    if ((pg.status() & ImageObserver.ABORT) != 0) {
      System.err.println("ImageView: pixel grab aborted or errored");
      return false;
    }

    controller.newZoom(data);
    return true;

  }


  public boolean mouseMove(Event event, int x, int y) {

    if (x >= 0 && x < size.width && y >= 0 && y < size.height) {

      cursor_pos.x = x;
      cursor_pos.y = y;

      PixelGrabber pg = new PixelGrabber(image, x, y, 1, 1,data, 0, max_size);

      try {
        pg.grabPixels();
      }
      catch (InterruptedException e) {
        System.err.println("ImageView: interrupted while grabbing pixels");
        return false;
      }
      if ((pg.status() & ImageObserver.ABORT) != 0) {
        System.err.println("ImageView: pixel grab aborted or errored");
        return false;
      }

      controller.newSample(cursor_pos, data[0]);
      return true;

    }
    else
      return false;

  }


  public boolean mouseDown(Event event, int x, int y) {
    if (x >= margin && x < size.width - margin &&
        y >= margin && y < size.height - margin) {
      window_pos.x = x;
      window_pos.y = y;
      markPosition();
      return sampleWindow();
    }
    else
      return false;
  }


}
