/*
* Copyright (c) 2011 Massimo Cavalleri <massimo.cavalleri@gmail.com>
* Copyright (c) 2011 Mattia Andena <progmat2000@yahoo.it>
* Copyright (c) 2011 Alessandro Ceseno <ing@alexceseno.it>
*
* This file is part of TTEI.
*
* TTEI is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* TTEI is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with TTEI.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* <p>Title: Operatori per estrazione dei contorni dell'immagine </p>
* <p>Description:Sobel, Prewitt, Isotropico, Kirsch, Roberts, Difference Of Gaussian </p>
* <p>Copyright: Copyright (c) 2011</p>
* <p>License: GPL </p>
* @author Massimo Cavalleri, Mattia Andena, Alessandro Ceseno
* @version 1.0
*/
public class Op{

 private int w, h, max, min,ControlloBordo;

 private int Modulo[],accD[];
 
 double  DoG[],MatGauss[],Fase[];//MatGauss conterr l'elaborazione
                                 //in modulo o fase, non ancora ricampionata
                                 //ho dovuto definirle double, altrimenti i coefficienti
                                 //della matrice 21*21 sarebbero uguali a 0

 public int accH[],accV[];

 public Op(int width, int height) {        
                                           
    this.w = width;
    this.h = height;

  }
  
 //questo metodo seleziona il filtro o l'operatore da applicare all'imagine basandosi sulla 
 //richiesta proveniente dall'ImageView
 public void chiamataOp(int[] in,int[] out,int op, String mf)
  {
  	switch(op)
  	{
  		
               
  		case 1:Invert.invert(in, out,w,h);
                      break; 		
   		case 2:filtro(in,out,op,mf);//Sobel
   		             break;
   		case 3:filtro(in,out,op,mf); //Prewitt
   		               break;
   		case 4:filtro(in,out,op,mf);//Isotropico
   		                  break;
   		case 5:filtro(in,out,op,mf);//Kirsch
   		              break;
   		case 6:filtro(in,out,op,mf);//Roberts
  		               break;
  		case 7: DoG(in,out);
  		       
  		
  	}
  }

 //questo metodo applica le maschere dei filtri selezionate sulla matrice d'ingresso,
 //memorizzando i risultati in accumulatori orizzontali accH e verticali accV
 
 private void SetAcc(int[] in, int tipo)
  {
     int A,B,C,D,E,F,G,H,v=0;
     int dim= (w-2)*(h-2); //i due acc sono lunghi come la matrice in ingresso
     accH=new int[dim];    //ma senza i bordi che non vanno elaborati
     accV=new int[dim];
     accD=new int[dim];
   for(int i=1;i<h-1;i++)
    for(int j=1;j<w-1;j++)
      {
	   A = in[(i-1)*w+(j-1)];
	   B = in[(i-1)*w+j];
	   C = in[(i-1)*w+(j+1)];
	   D = in[i*w+(j-1)];
	   E = in[i*w+(j+1)];
	   F = in[(i+1)*w+(j-1)];
	   G = in[(i+1)*w+j];
	   H = in[(i+1)*w+(j+1)];
	   
	   switch(tipo)
	   {
	    case 2://Sobel
	           accH[v]=C+2*E+H-A-2*D-F;
	           accV[v]=F+2*G+H-A-2*B-C;
               break;

	    case 3: //Prewitt
	           accH[v]=C+E+H-A-D-F;
	           accV[v]=F+G+H-A-B-C;
               break;

	    case 4: //Isotropico
	           accH[v]=C+((int)Math.sqrt(2))*E+H-A-((int)Math.sqrt(2))*D-F;
	           accV[v]=F+((int)Math.sqrt(2))*G+H-A-((int)Math.sqrt(2))*B-C;
	           //System.out.println("isotropico");
               break;


	    case 5: //Kirsch
	           accH[v]=-3*(A+D+F+B+G)+5*(C+E+H);
	           accV[v]=-3*(A+B+C+D+E)+5*(F+G+H);
               break;
               
        case 6://Roberts
	           accH[v]=-G+E;
	           accV[v]=in[i*w+j]-H;
	           break;
	     
	    case 7: //DoG:questo non  il metodo utilizzato della classe; abbiamo voluto lasciarlo
	            //a scopo dimostrativo:con questo metodo si filtrava l'immagine prima con una
	            //maschera 3*3 e poi con una maschera 5*5,per non sacrificare i pixel di bordo
	            //dell'immagine 
	           if((i==1)||(j==1)||(i==h-2)||(j==w-2))
	          	 accD[v]=-B-D+4*in[i*w+j]-E-G;
	           else
                {
                 int I1=in [(i-2)*w+j];
                 int I2=in [i*w+(j-2)];
                 int I3=in [i*w+(j+2)];
                 int I4=in [(i+2)*j];
                 accD[v]=-2*(B+D+E+G)-1*(A+C+F+H+I1+I2+I3+I4)+16*in[i*w+j];
                }
        }
             
        v++;
      }


  }

 //calcola il massimo del modulo al fine di effettuare la normalizzazione
 public void Massimo(int[] in)
 {
   max=in[w+1];
   for(int i=1;i<h-1;i++)
     for(int j=1;j<w-1;j++)
       if(max<in[i*w+j])
        max=in[i*w+j];
 }//chiusura Massimo
      
 //calcola il minimo del modulo al fine di effettuare la normalizzazione
 public void Minimo(int[] in)
 {
   min=in[w+1];
   for(int i=1;i<h-1;i++)
     for(int j=1;j<w-1;j++)
       if(min>in[i*w+j])
        min=in[i*w+j];
 }//chiusura minimo
      
 //questo metodo effetua la normalizzazione del modulo, portando i valori dei pixel che 
 //compongono l'immagine d'uscita nell'intervallo compreso tra 0 e 255
 public void Normalizzazione(int[] Start,int[] out)
 {
   Massimo(Start);
   Minimo(Start);
         
   for(int i=1;i<h-1;i++)
     for(int j=1;j<w-1;j++)
       out[i*w+j]=(int)(255*(double)(Start[i*w+j]-min)/(double)(max-min));
 }

  //questo metodo calcola il modulo o la fase ( dipende dalla richiesta proveniente dall
  //ImageView)
  public void filtro(int[] in, int[] out, int tipo,String mf)
  {
    // fa una verifica di errore
    if(in.length != w*h || out.length != w*h)
    {
      System.err.println("Op: le dimensioni delle immagini non sono corrette!");
      return;
    }
    
    Modulo=new int [w*h];
    Fase = new double[w*h];
    
    //azzera i bordi dei seguenti vettori
    for(int i=0;i<h;i++)
      for(int j=0;j<w;j++)
       {out[i*w+j]=0;
        Modulo[i*w+j]=0;
        Fase[i*w+j]=0;
       }

    SetAcc(in,tipo);

    int v=0;
    if(mf.equals("modulo"))
     { 
       for(int i=1;i<h-1;i++)
         for(int j=1;j<w-1;j++)
          {//calcola il modulo dei vari filtri eccetto il DoG
          	Modulo[i*w+j] = (int) Math.sqrt( Math.pow((accV[v]),2.0)+Math.pow((accH[v]),2.0) );
	       v++;
          }
	   Normalizzazione(Modulo,out);
	   
	   /*for(int i=1;i<h-1;i++)
         for(int j=1;j<w-1;j++)
           out[i*w+j]=Modulo[i*w+j];*/
     }
    else 
     { //fase
        
        for(int i=1;i<h-1;i++)
         for(int j=1;j<w-1;j++)
          {Modulo[i*w+j] = (int) Math.sqrt( Math.pow((accV[v]),2.0)+Math.pow((accH[v]),2.0) );
	       v++;
          }
          
	    Normalizzazione(Modulo,Modulo);
	    v=0;
	    
	    for(int i=1;i<h-1;i++)
         for(int j=1;j<w-1;j++)
          { 
	        //calcola la fase dei vari filtri eccetto il DoG
	        Fase[i*w+j]=Math.atan2((double) accV[v],(double) accH[v]);
	                                                                   
            if(tipo == 6)
              Fase[i*w+j] += Math.PI/4; 
            
            if(Fase[i*w+j]<0)
              Fase[i*w+j]=(Fase[i*w+j]+Math.PI);
	        //normalizzo la fase
            out[i*w+j]=(int) ((double) Fase[i*w+j]*255.0/Math.PI);   
            
            //applico una soglia alla fase in modo tale da diminuire i rumori(a nostro avviso 30  il valore
            //ottimale, perch, se aumentiamo la soglia, l'immagine  carente di dettagli ed alcuni
            //contorni rimangono aperti, se diminuiamo la soglia, l'immagine  colma di rumori)
            if(Modulo[i*w+j]<30)
              out[i*w+j]=0;
            
            v++;
	      }   
	            
     }



 }
 
public void FasePura(int[] in, double[] out) //questo metodo  utilizzato solo per la ricerca dei poligoni  
                                             //contiene la fase non normalizzata, non sogliata
                                             //che varia da -PI a PI
  {
    // fa una verifica di errore
    if(in.length != w*h || out.length != w*h)
    {
      System.err.println("Op: le dimensioni delle immagini non sono corrette!");
      return;
    }
    
    Fase = new double[w*h];
    
    
    for(int i=0;i<h;i++)
      for(int j=0;j<w;j++)
       {
       	out[i*w+j]=0;
        Fase[i*w+j]=0;
       }

    SetAcc(in,4);
    
    int v=0;
    
       //System.out.println("Fase Pura");
      
       for(int i=0;i<h;i++)
        for(int j=0;j<w;j++)
         Fase[i*w+j]=0;
         
       for(int i=1;i<h-1;i++)
        for(int j=1;j<w-1;j++)
         {
	      
	      Fase[i*w+j]=Math.atan2((double) accV[v],(double) accH[v]);	
          
          out[i*w+j] = Fase[i*w+j];
          v++;
          
         }
      
 }     
 
//Questo  il metodo che effettivamente viene utilizzato dalla classe
public void DoG(int[] in, int[] out)
 {
   DoG=new double [w*h];
   if(in.length != w*h || out.length != w*h)
    {//apre verifica
      System.err.println("Op: le dimensioni delle immagini non sono corrette!");
      return;
    }//chiude verifica

   for(int i=0;i<h;i++)
      for(int j=0;j<w;j++)
       {DoG[i*w+j]=0;
       out[i*w+j]=0;
       }//chiude for

      /*Ho preferito sacrificare il numero dei pixel saltando
       i primi 10 pixel dei bordi dell'immagine.(Il
       sigma(calcolato empiricamente) della matrice 21*21,non era compatibile 
       con le altre; avrei dovuto calcolarmi sperimentalmente un sigma per ogni
       matrice dei primi 10 pixel).*/

      //indica le dimensione della matrice
      int MAX=21;
      
   //queste 2 righe di commento facevano parte del secondo  codice scritto precedentemente
   //il cui compito era di filtrare l'immagine iniziale applicando rispettivamente maschere 
   //da 3*3,5*5,7*7,...,21*21, per non disperdere i pixel di bordo; ma tale metodo risult
   //fallimentare, in quanto avremmo dovuto calcolarci sperimentalmente ogni valore di sigma1 
   // e sigma2 per ogni maschera utilizzata 
   //for(int n=3;n<23;n+=2){
   //MAX=n;
      int v;
      MatGauss(MAX);
   for(int i=(MAX-1)/2;i<h-((MAX-1)/2);i++)
     for(int j=(MAX-1)/2;j<w-((MAX-1)/2);j++)
       {v=0;
        //applico la maschera determinata dalle differenze delle gaussiane sul vettore d'ingresso
       	for(int a=i-((MAX-1)/2);a<i+((MAX+1)/2);a++)
         for(int b=j-((MAX-1)/2);b<j+((MAX+1)/2);b++)
          { 
          //il valore del pixel corrente del vettore dog  la somma dei suoi intorni nel vettore
          //d'ingresso moltiplicati per le rispettive differenze delle gaussiane 
          DoG[i*w+j]+=in[a*w+b]*MatGauss[v];
               v++;
          }
       }

         //assegna al pixel corrente del vettore d'uscita il valore 1 se quello stesso punto di
         //DoG  minore di zero, il valore 0 se quello stesso punto di Dog  >0  
         for(int i=(MAX-1)/2;i<h-(MAX-1)/2;i++)
           for(int j=(MAX-1)/2;j<w-(MAX-1)/2;j++)
           {
           if(DoG[i*w+j]<0)
              out[i*w+j]=1;
	    else
               out[i*w+j]=0;
             }

           for (int i=(MAX-1)/2;i<h-(MAX-1)/2;i++)
              for(int j=(MAX-1)/2;j<w-(MAX-1)/2;j++)
               {ControlloBordo(out[i*w+j],out,i,j,MAX);
                 out[i*w+j]=ControlloBordo;
                }//for

   //}
 }
   //controlla se ogni punto dell'immagine rappesenti o non rappresenti il bordo della stessa
   public void ControlloBordo(int outAtt, int[] out, int a, int b,int MAX)
   {
     ControlloBordo=0;
    for(int i=a-(MAX-1)/2;i<a+(MAX+1)/2;i++)
         for(int j=b-(MAX-1)/2;j<b+(MAX+1)/2;j++)
      {
       //outAtt significa valore del vettore di uscita corrente
       if(outAtt==1){
        if(outAtt!=out[i*w+j]){
           if(DoG[i*w+j]-DoG[a*w+b]>0)//soglia optional del DoG
             ControlloBordo=255;
        }
       }

    }

   }
   //calcola la differenza delle gaussiane
   public void MatGauss(int MAX )
   {
        MatGauss=new double[MAX*MAX];
        double Sigma1,Sigma2;
        Sigma1=2.0;//per quanto concerne la scelta di questi valori, 
        Sigma2=4.0;//vedere nel sito la sezione conclusioni
        for(int i=0;i<MAX;i++)
           for(int j=0;j<MAX;j++)
             {
              //azzero i bordi
              if((i!=(MAX-1)/2 && (j==0 || j==MAX-1)) || (j!=(MAX-1)/2 && (i==0 || i==MAX-1))) 
           	MatGauss[i*MAX+j]=0;
           	//determino le differenze delle gaussiane
           	else{MatGauss[i*MAX+j]= ((1 / (2 * Math.PI * Sigma1*Sigma1)) * Math.exp(-1 * ((
				(i-(MAX-1)/2)*(i-(MAX-1)/2) + (j-(MAX-1)/2)*(j-(MAX-1)/2)) / (2 *
				Sigma1*Sigma1))));
           
				MatGauss[i*MAX+j] -= ((1 / (2 * Math.PI * Sigma2*Sigma2)) * Math.exp(-1 * ((
				(i-(MAX-1)/2)*(i-(MAX-1)/2) + (j-(MAX-1)/2)*(j-(MAX-1)/2)) / (2 *
				Sigma2*Sigma2))));          
				}
           
           }  
  }


}





