/*
* Copyright (c) 2011 Massimo Cavalleri <massimo.cavalleri@gmail.com>
* Copyright (c) 2011 Mattia Andena <progmat2000@yahoo.it>
* Copyright (c) 2011 Alessandro Ceseno <ing@alexceseno.it>
*
* This file is part of TTEI.
*
* TTEI is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* TTEI is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with TTEI.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* <p>Title: Implementazione trasformata di Hough per il riconoscimento di Poligoni regolari </p>
* <p>Description:Cerchi, Triangoli, Rettangoli, Quadrati, Esagoni e Ottagoni </p>
* <p>Copyright: Copyright (c) 2011</p>
* <p>License: GPL </p>
* @author Massimo Cavalleri, Mattia Andena, Alessandro Ceseno
* @version 1.0
*/
class Poligoni
{ 
 private int w, h;
 int lato,VotiMax,VotiMin;
 double apotema;                                          
 int[] Voti;
 double Theta[];
 int modulo[];
 Op FM; 
 
 public Poligoni(int width, int height) 
  {                                         
    this.w = width;
    this.h = height;
    VotiMin=0;
    Voti=new int[w*h];
    Theta=new double[w*h];
    modulo=new int[w*h];
    FM = new Op( w, h );
  }
  
  public void hough(int in[], int out[], String uscita, int figura)
   {
   	//System.out.println("Sono dentro Poligoni");
   	
   	   FM.chiamataOp(in,modulo,4,"modulo");
   	   FM.FasePura(in,Theta);
   	
   	
    //System.out.println(figura);  	
   	  
   	   for(int i=0;i<h;i++)
         for(int j=0;j<w;j++)
         {
           out[i*w+j]=0;
           Voti[i*w+j]=0;   
         }
      
       SetFigura(figura); 
      
       RicercaFigura();
       
       imageOut(uscita, in, out, figura);
       
         
   }     
   
 public void imageOut(String uscita, int in[], int out[], int figura)
 {
 	if(uscita.equals("immagine di uscita"))
 	{
 	   for(int i=0;i<h;i++)
        for(int j=0;j<w;j++)
            out[i*w+j] = in[i*w+j];
      
      //trovo il numero di voti massimo
      VotiMax=0;
      
      for(int i=0;i<h;i++)
       for(int j=0;j<w;j++)
         if(Voti[i*w+j]>VotiMax)
         	VotiMax = Voti[i*w+j]; 
 
     
     int Massimo = 0, somma;
     
     //uso la somma pesata di un intorno per dare maggior peso ai pixel pi 'concentrati'
     //della matrice Voti
     // trovo il valore della somma pesata pi grande e lo metto in Massimo
     for(int y = 3; y < h-4; y++)
	   for(int x = 3; x < w-4; x++)
		{
			if(Voti[y*w+x]>=(0.6*VotiMax))
			 {	
			    somma=0;

				for(int i = -3; i <= 3; i++)
				  for(int j = -3; j <= 3; j++)
					{
						if(i==0 && j==0)
						 somma += 16*Voti[(y+i)*w+(x+j)];
						else
						  if(i < 2 && i > -2 && j < 2 && j > -2)
						   somma += 4*Voti[(y+i)*w+(x+j)];
						  else
						   somma += Voti[(y+i)*w+(x+j)];
						
					}
				
				if(somma > Massimo)
					Massimo = somma;
		    }
		}
		

		for(int y = 3; y < h-4; y++)
		  for(int x = 3; x < w-4; x++)
			{
				somma=0;

				for(int i = -3; i <= 3; i++)
				  for(int j = -3; j <= 3; j++)
					{
						if(i==0 && j==0)
						  somma += 16*Voti[(y+i)*w+(x+j)];
						else
						  if(i < 2 && i > -2 && j < 2 && j > -2)
						    somma += 4*Voti[(y+i)*w+(x+j)];
						  else
						    somma += Voti[(y+i)*w+(x+j)];	
					}
				

                double perc=0.95;
                if((figura == 4)||(figura==1)) perc = 0.65;
                
				if(somma >= perc*Massimo)
				 {
					somma=0;

                    //disegno la croce
					for (int k = -2; k <= 2; k++) 
                      { 
		               out[(y+k)*w+(x+k)] = 0;
		               out[(y+k)*w+(x-k)] = 0;
	                  }	
				 }
			}      
      	  
 	}
 	else
 	 if(uscita.equals("spazio dei parametri"))
 	 {
 	   normalizzaVoti();
 	   
 	   for(int i=0;i<h;i++)
        for(int j=0;j<w;j++)
            out[i*w+j] = Voti[i*w+j];
 	 }
 	
 }
   
 public void normalizzaVoti()
  {
    VotiMax=0;
    VotiMin=0;

    for(int i=0;i<h;i++)
     for(int j=0;j<w;j++)
       if(Voti[i*w+j]>VotiMax)
         	VotiMax = Voti[i*w+j];
         

    for(int i=0;i<h;i++)
     for(int j=0;j<w;j++)
      if(Voti[i*w+j]!=0)
        Voti[i*w+j]=(255*(Voti[i*w+j]-VotiMin)/(VotiMax-VotiMin));
  }   


   public void SetFigura(int figura)
    {
      switch(figura)
      {
        case 0://cerchi piccoli
		       lato=0;
		       apotema=25;
		       System.out.println("C P");
		       break;
    
        case 1://cerchi medi
		       lato=0;
		       apotema=35;
		       System.out.println("C M");
		       break;
		       
	    case 2://cerchi grandi
		       lato=0;
		       apotema=50;
		       System.out.println("C G");
		       break;
		       
	    case 3://triangoli
		       lato=60;
		       apotema=lato*0.289;
		       System.out.println("T");
		       break;
	
	    case 4://quadrati piccoli
		       lato=40;
		       apotema=lato/2;
		       System.out.println("Q P");
		       break;
		       
	    case 5://quadrati medi
	           lato=60;
	           apotema=lato/2;
	           System.out.println("Q M");
	           break;
	           
	    case 6://quadrati grandi
		       lato=89;
		       apotema=lato/2;
		       System.out.println("Q G");
		       break;
		       
	    case 7://rettangolo 
	           lato=20;
	           apotema=lato/2;
	           break;

	    case 8://esagoni piccoli
		       lato=30;
		       apotema=lato*0.866;
		       System.out.println("E");
		       break;
	
	    case 9://esagoni grandi
	           lato=60;
	           apotema=lato*0.866;
	           System.out.println("E G");
	           break;
	
	    case 10://ottagoni
		        lato=18;
		        apotema=lato*1.207;
		        System.out.println("O");
		
	}
    
  }  

  public void RicercaFigura()
  {  
   
   double ThO,Th;
   
   /*for(int i=0;i<h;i++)
    for(int j=0;j<w;j++)
     System.out.println(Theta[i*w+j]);*/
   
   for(int i=0;i<h;i++)
    for(int j=0;j<w;j++)
      if(modulo[i*w+j]!=0)
      {
      	//System.out.println("i:" + i + " j:" + j + " T:" + Theta[i*w+j]); 
      		
      	Th = -Math.PI -Theta[i*w+j]; //ATTENZIONE
        ThO =  Th + (Math.PI / 2);
      	
      	int ic = (int) (i + (Math.sin(Th) * apotema));
      	int jc = (int) (j - (Math.cos(Th) * apotema));
      		
      	for(int dl=-lato/2;dl<=(lato/2);dl++)
      	 {
      	 	int iv = (int)(ic - (dl*Math.sin(ThO)));
      	 	int jv = (int)(jc + (dl*Math.cos(ThO)));
      	 	
      	 	if( (iv>=0) && (iv<h) && (jv>=0) && (jv<w) )
      	 	 Voti[iv*w+jv]++;
      	 }
      	
      }
     
  }        
       
   
}   
