/*
* Copyright (c) 2011 Massimo Cavalleri <massimo.cavalleri@gmail.com>
* Copyright (c) 2011 Mattia Andena <progmat2000@yahoo.it>
* Copyright (c) 2011 Alessandro Ceseno <ing@alexceseno.it>
*
* This file is part of TTEI.
*
* TTEI is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* TTEI is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with TTEI.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* <p>Title:Implementazione della trasformata di Hough per il riconiscimento delle rette </p>
* <p>Copyright: Copyright (c) 2011</p>
* <p>License: GPL </p>
* @author Massimo Cavalleri, Mattia Andena, Alessandro Ceseno
* @version 1.0
*/
import java.util.*;
import java.math.*;

class Retta{
 
 private int w, h;
 final double RhoMax;
 int Voti[];
 double [] Theta;
 Op a;
                                           
 public Retta(int width, int height) {      
                                           
    this.w = width;
    this.h = height;
    a=new Op(w,h);
    RhoMax=Math.sqrt(Math.pow(w,2)+Math.pow(h,2));
    
  }
  
  public void hough(int in[], int out[],int n) 
   {
   	int intmp[];
   	intmp=new int[w*h];
   	
   	a.chiamataOp(in, intmp,4,"modulo"); // modulo isotropico
   	
   	for(int i=0;i<h;i++)
      for(int j=0;j<w;j++)
       out[i*w+j]=in[i*w+j];
       
   
   
    Theta=new double[314];
    
    //calcolo tutti i valori dell'asse Theta e memorizzo gli stessi in un vettore
    for(int i=0;i<314;i++)
    {
      Theta[i]=(Math.PI/314*i);
     
   // System.out.println(Theta[i]);
     
    }
      
   
   //Definisco Voti come il vettore dello spazio dei parametri
    Voti = new int[Theta.length*((int)(2*RhoMax))];
    int RhoCalc=0;
    for(int v=0;v<Voti.length;v++)
     Voti[v]=0; 
    
    //Cicli che incrementano il singolo pixel dello spazio dei parametri al variare di Rho e Theta
    for(int i=0;i<h;i++)
     for(int j=0;j<w;j++)
      { 
        if(intmp[i*w+j]>50) //(0  il nero) ho ritenuto opportuno l'uso di questa soglia per
        {                  // un discreto compromesso tra velocit di esecuzione e minimo numero
                           //di pixel necessario per costruire lo spazio dei parametri
       
         for(int k=0;k<Theta.length;k++) //ciclo per il fascio: theta varia
          {
           RhoCalc=(int) (j*Math.cos(Theta[k])+i*Math.sin(Theta[k])); 
           
           RhoCalc+=RhoMax;//Sommo RhoMax per riportare RhoCalc tra 0 e 2RhoMax
           
           Voti[k*((int)(2*RhoMax))+RhoCalc]++;
          }
         
        }
         
       //System.out.println("pixel:" + (i*w+j));
      }  
      
    
    
     //troviamo le coordinate nello spazio dei parametri di n massimi
   
    //n rappresenta il numero dei massimi locali ma anche il numero di rette che
    //sucessivamente disegneremo nel piano xy
   
     //Vettore che memorizza le coordinate Rho e Theta di ogni massimo locale
     double Ctr[][]=new double[2][n];
     for (int i=0;i<2;i++)
       for(int j=0;j<n;j++)
         Ctr[i][j]=0;

     double VotiM;
     boolean b;
     
     for(int k=0;k<n;k++) //k punta la posizione libera del Ctr
      {
      	VotiM=-1;
      	
        for(int vi=0;vi<Theta.length;vi++)
          for(int vj=0;vj<((int)(2*RhoMax));vj++)
           if(VotiM < Voti[vi*((int)(2*RhoMax))+vj])
            {
             b=true;
           
             for(int j=0;(j<k)&&(b!=false);j++) //controlla se c' gi nelle posizioni prec
              {
                if((Ctr[0][j]==Theta[vi]) && (Ctr[1][j]==vj))	
          	  	 b=false;
          	    else
          	    {
          	     //controlla che Voti[vi*((int)(2*RhoMax))+vj] non faccia parte di un 
          	     //intorno 5x5 di uno dei massimi locali	
          	     double dt;
          	     int dr;

          	       for(int t=-1;(t<2)&&(b!=false);t++)
          	        for(int r=-1;(r<2)&&(b!=false);r++)
          	        {
          	         if((r==0)&&(t==0)) continue;
          	         dt=(int)(((Ctr[0][j])/(Math.PI/314))+t);
          	         dr=((int) Ctr[1][j])+r;
          	         if(dt<0 || dt>=Theta.length) continue;
          	         if(dr<0 || dr>=(2*RhoMax)) continue;
          	         if((dt==vi)&&(dr==vj))
          	           b=false;
          	           
          	        }
          	    }
          	    
              }
          	                             
             if(b==true)                                  
          	  {
          	   VotiM=Voti[vi*((int)(2*RhoMax))+vj];
          	   Ctr[0][k]=Theta[vi];
               Ctr[1][k]=vj; 
          	  }	 
          
            }
         
       
       
     }

    /* for (int i=0;i<2;i++)
      {System.out.println(); 
       for(int j=0;j<n;j++)
         System.out.print(Ctr[i][j] + " ");
       }*/
   
     //dalle coordinate dei massimi espresse nello spazio dei parametri, torniamo nel piano 
     // xy per trovare le coordinate dei punti della retta che era rappresentata nel piano
     //Rho Theta dal punto di massimo 
      	  
           for(int k=0;k<n;k++)
              { 
                   if(( Ctr[0][k] >= (Math.PI/4) ) && ( Ctr[0][k] <= (3*Math.PI/4) ))
                       for(int x=0;x<w;x++)
                       {
                       	int y;
                        y=(int)(((Ctr[1][k]-RhoMax)-x*( Math.cos( Ctr[0][k] )))/Math.sin( Ctr[0][k] ));
         
                        if((y>=0)&&(y<h))  //controllo di rimanere all'interno delle dimensioni dell'immagine
                         out[y*w+x]=255; 
                       }   
                   else
                       for(int y=0;y<h;y++)
                        {
                         int x;
           	             x=(int)(((Ctr[1][k]-RhoMax)-y*( Math.sin( Ctr[0][k] )))/Math.cos(Ctr[0][k]));
           	
           	             if((x>=0)&&(x<w))
                           out[y*w+x]=255;
                        }
             }
      
     
       
     System.out.println("finito!");
  }
  
 public void disegnaSp()
 {
  normalizzaVoti();       
  SpView SP = new SpView(Voti,((int)(2*RhoMax)),Theta.length);
 }        
  
 public void normalizzaVoti()
  {
    int VotiMax=0;
    int VotiMin=0;

    for(int i=0;i<Theta.length;i++)
     for(int j=0;j<((int)(2*RhoMax));j++)
       if(Voti[i*((int)(2*RhoMax))+j]>VotiMax)
         	VotiMax = Voti[i*((int)(2*RhoMax))+j];
         

    for(int i=0;i<Theta.length;i++)
     for(int j=0;j<((int)(2*RhoMax));j++)
       if(Voti[i*((int)(2*RhoMax))+j]!=0)
        Voti[i*((int)(2*RhoMax))+j]=(255*(Voti[i*((int)(2*RhoMax))+j]-VotiMin)/(VotiMax-VotiMin));
  }   
 
}
